#!/usr/bin/perl -w

# the input file format should be, e.g.,
## gene
## transcript
## exon
## exon
## transcript
## exon

use strict;
use Getopt::Long;

my $help;
my $input;
my $output;

my $line;
my $linetsc;
my @args;
my @maxlines;
my @lines;
my $maxlen;
my $len;
my $maxnumexon;
my $numexon;

usage() if (@ARGV < 2 or
            ! GetOptions("help|?" => \$help,
			 "input=s" => \$input,
			 "output=s" => \$output)
	    or defined $help);

if (defined $input) {
    open(STDIN, $input);
}
if (defined $output) {
    open(STDOUT, ">$output");
}

$line = <STDIN>;

do {
    # jump "gene" feature
    $line = <STDIN>;
    if ($line) {
	chomp($line);
	@args = split("\t", $line);
    }
    @lines = ();
    $maxlen = 0;
    $maxnumexon = 0;
    while ($args[2] eq "transcript") {
	$linetsc = $line;
	$numexon = 0;
	$len = $args[4] - $args[3];
	
	$line =~ s/transcript/gene/g;
	$line =~ s/Parent/Name/g;
	$line =~ s/parent/Name/g;
	@lines = ();

	do {
	    push(@lines, $line);
	    $line = <STDIN>;
	    if ($line) {
		chomp($line);
		@args = split("\t", $line);
		$numexon += 1;
	    }
	} while ($line && $args[2] eq "exon");  
	
	if ($numexon > $maxnumexon || ($numexon==$maxnumexon && $len > $maxlen)) {
	    $maxlen = $len;
	    $maxnumexon = $numexon;
	    @maxlines = @lines;
	}
    }
    print STDOUT join("\n", @maxlines);
    print STDOUT "\n";
} while ($line);

close(STDIN);
close(STDOUT);
exit 0;

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print <<enddocs
Get the most spliced isoform when there are several isoforms for a gene in a given gff file.

USAGE: perl getMostSpicedIsoform.pl -i|--input <IN_GFF_FILE> [OPTIONS]

OPTIONS:
    -h,--help,?
        Print this help

    -o,--output <OUT_GFF_FILE>
        Output file name

enddocs
;
    exit 0;
}
