#!/usr/bin/env perl
#!/bin/env perl

## Name: joinpairbed12.pl
## Version: 1.2.6
## Date: May 17, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

# join paired-end mates
# --input, input bed12 file
# --output, output bed file
# --junction, intron list file
## by default, print spliced segments from joined reads into output
## --junction, print joined reads into output and print introns into intron list

use strict;
use warnings;
use Getopt::Long;

my $help;
my $input;
my $junction;
my $output = "output.bed";
my $line1;
my $line2;
my @args1;
my @args2;
my @pos1;
my @pos2;
my @pos;
my @newpos;
my @blocksize;
my @blockpos;
my @old;
my @new;
my @fus;
my $i;
my $maxlen;
my $exonnum;
my $exonpos;
my $exonsize;
my $strand;
my $genestart;
my $geneend;

usage() if (@ARGV < 2 or
	    ! GetOptions("help|?" => \$help,
			 "input=s" => \$input,
			 "junction=s" => \$junction,
			 "output=s" => \$output)
	    or defined $help);

if (defined $input) {
    open(STDIN, $input);
}
open(OUT, ">$output");

if ($junction) {
    open(INTRON, ">$junction");
}

while ($line1=<STDIN>) {
    @pos1 = ();			# exon positions in the first mate
    @pos2 = ();			# exon positions in the second mate
    @pos = ();			# all exon postions

    #< first mate
    chomp($line1);
    @args1 = split("\t", $line1);
    #>
    #< second mate
    $line2=<STDIN>;
    chomp($line2);
    @args2 = split(" ", $line2);  
    #>
    
    #< first mate
    @blocksize = split(",",$args1[10]);
    if ($junction && $args1[2]<$args2[1]) {
	$blocksize[$#blocksize] = $blocksize[$#blocksize] + $args2[1] - $args1[2];
    }
    @blockpos = split(",",$args1[11]);
    for ($i=0; $i<$args1[9]-1; $i++) {
	push(@pos1, $args1[1]+$blockpos[$i]+$blocksize[$i], $args1[1]+$blockpos[$i+1])
    }
    $strand = $args1[5];
    #>
    
    #< second mate
    @blocksize = split(",",$args2[10]);
    if ($junction && $args2[2] < $args1[1]) {
	$blocksize[$#blocksize] = $blocksize[$#blocksize] + $args1[1] - $args2[2];
    }
    @blockpos = split(",",$args2[11]);
    for ($i=0; $i<$args2[9]-1; $i++) {
	push(@pos2, $args2[1]+$blockpos[$i]+$blocksize[$i], $args2[1]+$blockpos[$i+1])
    }
    #>
    
    #print "@pos1\n";
    #print "@pos2\n";
    
    
    #< merge pos1 and pos2
    $maxlen = max($#pos1+1, $#pos2+1)/2;
    while ($maxlen>0) {
	if ($#pos2 < 0) {
	    while ($#pos1>0) {
		push(@pos, $pos1[0]);
		shift(@pos1);
		push(@pos, $pos1[0]);
		shift(@pos1);
	    }
	}
	elsif ($#pos1 < 0) {
	    while ($#pos2>0) {
		push(@pos, $pos2[0]);
		shift(@pos2);
		push(@pos, $pos2[0]);
		shift(@pos2);
	    }
	}
    	elsif ($pos1[0]<$pos2[0]) {
    	    push(@pos, $pos1[0]);
    	    shift(@pos1);
    	    push(@pos, $pos1[0]);
    	    shift(@pos1);
    	}
    	else {
    	    push(@pos, $pos2[0]);
    	    shift(@pos2);
    	    push(@pos, $pos2[0]);
    	    shift(@pos2);
    	}
    	$maxlen = max($#pos1+1, $#pos2+1)/2;
    }
    #print "@pos\n";
    #>
    
    #< gene positions
    $genestart = min($args1[1], $args2[1]);
    $geneend = max($args1[2], $args2[2]);
    #>  

    #< fusion introns from pos
    @newpos = ($genestart);
    while ($#pos >= 3) {
	if ($pos[1] >= $pos[2]) {
	    @old = ($pos[0], $pos[1]);
	    @new = ($pos[2], $pos[3]);
	    @fus = fusion(\@old,\@new);
	    shift(@pos);
	    shift(@pos);
	    $pos[0] = $fus[0];
	    $pos[1] = $fus[1];
	}
	else {
	    push(@newpos, $pos[0]);
	    push(@newpos, $pos[1]);
	    shift(@pos);
	    shift(@pos);
	}
    }
    while ($#pos>0) {
	push(@newpos, $pos[0]);
	shift(@pos);
	push(@newpos, $pos[0]);
	shift(@pos);
    }
    #print "@newpos\n";
    #>
    
    push(@newpos, $geneend);
    #< print exons from joined read
    if (! $junction) {
	while ($#newpos>1) {
	    print OUT "$args1[0]\t$newpos[0]\t$newpos[1]\t$args1[3]\n";
	    shift(@newpos);
	    shift(@newpos);
	}
	print OUT "$args1[0]\t$newpos[0]\t$newpos[1]\t$args1[3]\n";
	shift(@newpos);
	shift(@newpos);
    }
    #>

    #< print joined read
    else {
	$exonnum = ($#newpos+1)/2;
	if ($#newpos==1) {
	    $exonsize = $geneend - $genestart;
	    print OUT "$args1[0]\t$genestart\t$geneend\t$args1[3]\t$args1[4]\t$strand\t$genestart\t$geneend\t255,0,0\t$exonnum\t$exonsize\t0\n";
	}
	else {
	    #$exonsize = $newpos[0] - $genestart;
	    print OUT "$args1[0]\t$genestart\t$geneend\t$args1[3]\t$args1[4]\t$strand\t$genestart\t$geneend\t255,0,0\t$exonnum\t";
	    #print INTRON "$args1[0]\t$newpos[1]\t$newpos[2]\n";
	    
	    for ($i=0; $i<$#newpos-1; $i=$i+2) {
		$exonsize = $newpos[$i+1]-$newpos[$i];
		print OUT "$exonsize,";
		print INTRON "$args1[0]\t$newpos[$i+1]\t$newpos[$i+2]\n";
	    }
	    $exonsize = $geneend - $newpos[$i];
	    print OUT "$exonsize\t";
	    for ($i=0; $i<$#newpos-1; $i=$i+2) {
		$exonpos = $newpos[$i]-$genestart;
		print OUT "$exonpos,";
	    }
	    $exonpos = $newpos[$i]-$genestart;
	    print OUT "$exonpos\n";
	}
    }
    #>
}

close(STDIN);
close(OUT);
if ($junction) {
    close(INTRON);
}

exit 0;

sub max {
    if ($_[0] > $_[1]) {
	$_[0];
    }
    else {
	$_[1];
    }
}

sub min {
    if ($_[0] < $_[1]) {
	$_[0];
    }
    else {
	$_[1];
    }
}

sub fusion {
    my ($a, $b) = @_;
    my @ref = @{$a};
    #< take intron of smaller size
    if (${$a}[1] - ${$a}[0] <= ${$b}[1] - ${$b}[0]) {
	@ref = @{$a};
    }
    else {
	@ref = @{$b};
    }
    #>
    #$ref[0] = min(${$a}[0], ${$b}[0]);
    #$ref[1] = max(${$a}[1], ${$b}[1]);
    return @ref;
}

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: perl joinpairbed12.pl --input BED12 [--junction INTRON_OUT] [--output BED_OUT] [--help|?] \n";
    exit 1;
}
